###################################################
### chunk number 1: 
###################################################
  ## set the prompt to "> " and the continuation to "  "
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 2: 
###################################################
  library(chemometrics)


###################################################
### chunk number 3: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 4: 
###################################################
  library(gclus)                # contains the data set
  data(wine)
  X <- scale(wine[,2:14])       # first column: wine type


###################################################
### chunk number 5:  eval=FALSE
###################################################
##   res <- pcaCV(X)


###################################################
### chunk number 6: 01-pcaCV
###################################################
  par(mfrow=c(1,2))
  par(mar=c(5, 4, 1, 1))
  pcaCV(X)
  par(mar=c(7.5, 4, 1, 1))
  pcaVarexpl(X, a=5, las=2)


###################################################
### chunk number 7: 
###################################################
  X_nipals <- nipals(X, a=5)


###################################################
### chunk number 8: 
###################################################
  X_nipals <- nipals(X, a=5, it=160)


###################################################
### chunk number 9:  eval=FALSE
###################################################
##   res <- pcaVarexpl(X, a=5)


###################################################
### chunk number 10: 
###################################################
  X_nipals <- list(scores=X_nipals$T, loadings=X_nipals$P, 
    sdev=apply(X_nipals$T, 2, sd))


###################################################
### chunk number 11:  eval=FALSE
###################################################
##   res <- pcaDiagplot(X, X.pca=X_nipals, a=5)


###################################################
### chunk number 12: 03-pcaDiagplot-nipals
###################################################
  par(mar=c(5,4,1,1))
  res <- pcaDiagplot(X, X.pca=X_nipals, a=5)


###################################################
### chunk number 13:  eval=FALSE
###################################################
##   X.rob <- scale(wine[,2:14], center = apply(wine[,2:14], 2, median), 
##     scale = apply(wine[,2:14], 2, mad))
##   library(pcaPP)                 # robust PCA based on projection pursuit
##   X.grid <- PCAgrid(X.rob,k=5,scale=mad)
##   res1 <- pcaDiagplot(X.rob,X.grid,a=5)


###################################################
### chunk number 14: 04-pcaDiagplot-robust
###################################################
  X.rob <- scale(wine[,2:14], center = apply(wine[,2:14], 2, median), 
    scale = apply(wine[,2:14], 2, mad))
  library(pcaPP)                 # robust PCA based on projection pursuit
  X.grid <- PCAgrid(X.rob,k=5,scale=mad)
  par(mar=c(5,4,1,1))
  res1 <- pcaDiagplot(X.rob,X.grid,a=5)


###################################################
### chunk number 15: 
###################################################
# diagnostics with robust PCA
  orth1 <- dimnames(X)[[1]][(res1$ODist > res1$critOD) * (res1$SDist < res1$critSD) == 1] # orthogonal outliers
  good1 <- dimnames(X)[[1]][(res1$ODist < res1$critOD) * (res1$SDist > res1$critSD) == 1] # good leverage points
  bad1 <- dimnames(X)[[1]][(res1$ODist > res1$critOD) * (res1$SDist > res1$critSD) == 1] # bad leverage points
# diagnostics with NIPALS
  orth2 <- dimnames(X)[[1]][(res$ODist > res$critOD) * (res$SDist < res$critSD) == 1] # orthogonal outliers
  good2 <- dimnames(X)[[1]][(res$ODist < res$critOD) * (res$SDist > res$critSD) == 1] # good leverage points
  bad2 <- dimnames(X)[[1]][(res$ODist > res$critOD) * (res$SDist > res$critSD) == 1] # bad leverage points
# print:
  cat("DIAGNOSTICS WITH NIPALS:", "\n",
      "orthogonal outliers: ", orth2, "\n",
      "good leverage points:", good2, "\n",
      "bad leverage points: ", bad2, "\n", sep=" ")
  cat("DIAGNOSTICS WITH ROBUST PCA:", "\n",
      "orthogonal outliers: ", orth1, "\n",
      "good leverage points:", good1, "\n",
      "bad leverage points: ", bad1, "\n", sep=" ")


###################################################
### chunk number 16: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 17: 
###################################################
  data(NIR)
  X <- NIR$xNIR
  Y <- NIR$yGlcEtOH
  namesX <- names(X)
  attach(X)


###################################################
### chunk number 18:  eval=FALSE
###################################################
##   data(NIR)
##   X <- NIR$xNIR
##   Y <- NIR$yGlcEtOH
##   namesX <- names(X)
##   attach(X)


###################################################
### chunk number 19: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 20: 
###################################################
  y <- Y[,1]
  NIR.Glc <- data.frame(X=X, y=y)


###################################################
### chunk number 21:  eval=FALSE
###################################################
##   res.step <- stepwise(y~., data=NIR.Glc)


###################################################
### chunk number 22:  eval=FALSE
###################################################
##   steps    <- length(res.step$usedTime)
##   seconds  <- res.step$usedTime[steps]
##   varnbr   <- sum(finalvars <- res.step$models[steps,])
##   varnames <- namesX[as.logical(finalvars)]


###################################################
### chunk number 23: 
###################################################
  steps <- 22
  seconds <- 15
  varnbr <- 16
  varnames <- c('X1115.0','X1185.0','X1215.0','X1385.0','X1420.0','X1500.0','X1565.0','X1585.0',
                'X1690.0','X1715.0','X1720.0','X1815.0','X1995.0','X2070.0','X2100.0','X2195.0')


###################################################
### chunk number 24: 
###################################################
  cat(" steps needed:        ", steps, "\n",
      "seconds needed:      ", seconds, "\n",
      "number of variables: ", varnbr, "\n", sep=" ")


###################################################
### chunk number 25: 01-BIC eval=FALSE
###################################################
##   # produce Figure 5
##   modelsize <- apply(res.step$models, 1, sum)
##   plot(modelsize, res.step$bic, type="b", pch=20,
##     main="BIC during stepwise regression",
##     xlab="model size", ylab="BIC value")


###################################################
### chunk number 26: 01-BIC
###################################################
  # produce Figure 4.1
  modelsize <- c(1,2,3,4,5,6,7,8,9,10,11,10,11,12,11,12,13,14,15,16,15,16)
  res.step.bic <- c(1292.227,1231.984,1216.206,1182.616,1176.313,1172.719,1136.370,1075.659,
    1064.525,1060.948,1057.010,1053.532,1050.034,1047.147,1042.120,1039.618,
    1032.290,1023.446,1022.557,1022.301,1018.909,1017.506)
  plot(modelsize, res.step.bic, type="b", pch=20,
    main="BIC during stepwise regression",
    xlab="model size", ylab="BIC value")


###################################################
### chunk number 27: 
###################################################
  finalmodel <- as.formula(paste("y~", paste(varnames, collapse = "+"),
    sep = ""))
  res.stepcv <- lmCV(finalmodel, data=NIR.Glc, repl=100, segments=4)


###################################################
### chunk number 28: 02-lmCV
###################################################
  par(mfrow=c(1,2))
  plot(rep(y,100), res.stepcv$predicted, pch=".",
    main="Predictions from repeated CV",
    xlab="Measured y", ylab="Predicted y")
  abline(coef=c(0,1))
  points(y, apply(res.stepcv$predicted, 1, mean), pch=20)
  plot(res.stepcv$SEP, main="Standard Error of Prediction",
    xlab="Number of repetitions", ylab="SEP")
  abline(h=res.stepcv$SEPm)
  abline(h=median(res.stepcv$SEP), lty=2)


###################################################
### chunk number 29: 
###################################################
    cat(" SEP mean:               ", round(res.stepcv$SEPm, 3), "\n",
      "SEP median:             ", round(median(res.stepcv$SEP), 3), "\n",
      "SEP standard deviation: ", round(sd(res.stepcv$SEP), 3), "\n", sep=" ")


###################################################
### chunk number 30: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 31:  eval=FALSE
###################################################
##   res.pcr <- mvr_dcv(y~., data=NIR.Glc, ncomp=40, method = "svdpc",
##     repl = 100)


###################################################
### chunk number 32: 03-comppcr eval=FALSE
###################################################
##   plotcompmvr(res.pcr)


###################################################
### chunk number 33: 04-seppcr eval=FALSE
###################################################
##   optpcr <- res.pcr$afinal
##   plotSEPmvr(res.pcr, optcomp=optpcr, y=y, X=X, method="svdpc")


###################################################
### chunk number 34: 
###################################################
  optpcr <- 31


###################################################
### chunk number 35: 05-predpcr eval=FALSE
###################################################
##   plotpredmvr(res.pcr, optcomp=optpcr, y=y, X=X, method="svdpc")


###################################################
### chunk number 36: 06-respcr eval=FALSE
###################################################
##   plotresmvr(res.pcr, optcomp=optpcr, y=y, X=X, method="svdpc")


###################################################
### chunk number 37: 
###################################################
    #SEP <- apply(res.pcr$resopt[,1,], 2, sd)
    cat(" optimal number of PCs:  ", optpcr, "\n",
         "SEP mean:               7.433 \n",                      #round(mean(SEP), 4)
         "SEP median:             7.449 \n",                     #round(median(SEP), 4)
         "SEP standard deviation: 0.373 \n", sep=" ")    # round(sd(SEP), 4)


###################################################
### chunk number 38: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 39:  eval=FALSE
###################################################
##   res.pls <- mvr_dcv(y~., data=NIR.Glc, ncomp=40, method = "simpls",
##     repl = 100)


###################################################
### chunk number 40: 07-comppls eval=FALSE
###################################################
##   plotcompmvr(res.pls)


###################################################
### chunk number 41: 08-seppls eval=FALSE
###################################################
##   optpls <- res.pls$afinal
##   plotSEPmvr(res.pls, optcomp=optpls, y=y, X=X, method="simpls")


###################################################
### chunk number 42: 
###################################################
  optpls <- 14


###################################################
### chunk number 43: 09-predpls eval=FALSE
###################################################
##   plotpredmvr(res.pls, optcomp=optpls, y=y, X=X , method="simpls")


###################################################
### chunk number 44: 10-respls eval=FALSE
###################################################
##   plotresmvr(res.pls, optcomp=optpls, y=y, X=X, method="simpls")


###################################################
### chunk number 45: 
###################################################
    #SEP <- apply(res.pls$resopt[,1,], 2, sd)
    cat(" optimal number of PCs:  ", optpls, "\n",
         "SEP mean:                6.489 \n", #round(mean(SEP), 4)
         "SEP median:              6.491 \n", #round(median(SEP), 4)
         "SEP standard deviation:  0.408 \n", sep=" ") #round(sd(SEP), 4)


###################################################
### chunk number 46:  eval=FALSE
###################################################
##   res.pls1nipals <- pls1_nipals(X, y, a = res.pls$afinal)


###################################################
### chunk number 47:  eval=FALSE
###################################################
##   Ysc <- scale(Y)
##   res.pls2nipals <- pls2_nipals(X, Ysc, a = 9)


###################################################
### chunk number 48:  eval=FALSE
###################################################
##   a <- min(dim(X)[1], dim(X)[2], dim(Y)[2])
##   res.plseigen <- pls_eigen(X, Ysc, a = a)


###################################################
### chunk number 49: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 50: 11-prmcv eval=FALSE
###################################################
##   res.prmcv <- prm_cv(X, y, a = 40, opt = "median")


###################################################
### chunk number 51: 
###################################################
  oc <- 20
  cat(" optimal number of PCs: 20 \n",  #oc <- res.prmcv$optcomp
       "                         classic   20% trimmed \n",
       "SEP mean:                5.454      5.094 \n",          #round(mean(res.prmcv$SEPj[,oc]), 4), round(mean(res.prmcv$SEPtrimj[,oc]), 4)
       "SEP median:              5.488      5.070 \n",          # round(median(res.prmcv$SEPj[,oc]), 4), round(median(res.prmcv$SEPtrimj[,oc]), 4)
       "SEP standard deviation:  0.627      1.289 \n", sep=" ") #round(sd(res.prmcv$SEPj[,oc]), 4), round(sd(res.prmcv$SEPtrimj[,oc]), 4)


###################################################
### chunk number 52: 12-plotprm eval=FALSE
###################################################
##   plotprm(res.prmcv, y)


###################################################
### chunk number 53:  eval=FALSE
###################################################
##   prm(X, y, a = res.prmcv$optcomp, opt = "l1m", usesvd = TRUE)


###################################################
### chunk number 54: 11a-prmdcv eval=FALSE
###################################################
##   res.prmdcv <- prm_dcv(X, y, a = 40, opt = "median",repl=20)


###################################################
### chunk number 55: 07a-compprmdcv eval=FALSE
###################################################
##   plotcompprm(res.prmdcv)


###################################################
### chunk number 56: 08a-sepprmdcv eval=FALSE
###################################################
##   plotSEPprm(res.prmdcv,res.prmdcv$afinal,y,X)


###################################################
### chunk number 57: 08b-predprmdcv eval=FALSE
###################################################
##   plotpredprm(res.prmdcv,res.prmdcv$afinal,y,X)


###################################################
### chunk number 58: 08c-resprmdcv eval=FALSE
###################################################
## plotresprm(res.prmdcv,res.prmdcv$afinal,y,X)


###################################################
### chunk number 59: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 60: 13-plotridge
###################################################
  res.plotridge <- plotRidge(y~., data=NIR.Glc, lambda=seq(0.5,10,by=0.05))


###################################################
### chunk number 61: 14-ridgecv eval=FALSE
###################################################
##   res.ridge <- ridgeCV(y~., data=NIR.Glc, lambdaopt=res.plotridge$lambdaopt,
##     repl=100)


###################################################
### chunk number 62: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 63: 15-lassocv eval=FALSE
###################################################
##   res.lasso <- lassoCV(y~., data = NIR.Glc, fraction = seq(0, 1, by = 0.05), 
##     legpos="top")


###################################################
### chunk number 64: 16-lassocoef eval=FALSE
###################################################
##   res.lassocoef <- lassocoef(y~., data=NIR.Glc, sopt=res.lasso$sopt)


###################################################
### chunk number 65: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 66: 
###################################################
  SEP.stepwise <- 4.61 #res.stepcv$SEPm
  SEP20.stepwise <- 4.40 #mean(apply(resid.trim.stepwise, 2, sd))
  SEP.pcr <- 7.43 #mean(apply(res.pcr$resopt[,1,], 2, sd))
  SEP20.pcr <- 7.37 #mean(apply(resid.trim.pcr, 2, sd))
  SEP.pls <- 6.49 #mean(apply(res.pls$resopt[,1,], 2, sd))
  SEP20.pls <- 6.52 #mean(apply(resid.trim.pls, 2, sd))
  SEP.prm <- 5.40 #res.prmcv$SEPall[res.prmcv$optcomp]
  SEP20.prm <- 4.95 #res.prmcv$SEPop
  SEP.prmdcv <- 5.95 #res.prmdcv$SEPall[res.prmcv$optcomp]
  SEP20.prmdcv <- 5.86 #res.prmcdv$SEPop
  SEP.ridge <- 5.37 #res.ridge$SEPm
  SEP20.ridge <- 5.32 #mean(apply(resid.trim.ridge, 2, sd))
  SEP.lasso <- 6.48 #res.lasso$SEP[(res.lasso$fraction==res.lasso$sopt)]
  SEP20.lasso <- 5.89 #compute trimmed SEP within lassoCV function (not implemented)
  cat(" PREDICTION PERFORMANCE",
    "\n Method      SEP     SEP20%     Nr. of Variables / Components ",
      "\n                                    ",
    "\n stepwise    ", round(SEP.stepwise, 2), "    ", round(SEP20.stepwise, 2), "        ", varnbr, " variables",
    "\n PCR         ", round(SEP.pcr, 2), "    ", round(SEP20.pcr, 2), "       ", optpcr, " components",
    "\n PLS         ", round(SEP.pls, 2), "    ", round(SEP20.pls, 2), "       ", optpls, " components",
    "\n PRM-CV      ", round(SEP.prm, 2), "     ", round(SEP20.prm, 2), "       ", oc, " components",
    "\n PRM-DCV     ", round(SEP.prmdcv, 2), "    ", round(SEP20.prmdcv, 2), "       ", oc, " components",
    "\n Ridge       ", round(SEP.ridge, 2), "    ", round(SEP20.ridge, 2), "      ", dim(X)[2], " variables",
    "\n Lasso       ", round(SEP.lasso, 2), "    ", round(SEP20.lasso, 2), "      110 variables \n", #res.lassocoef$numb.nonzero
    sep="")


###################################################
### chunk number 67: 
###################################################
  cat(" COMPUTATION TIMES \n",
    "Method      Algorithm     Time needed \n \n",
    "stepwise    100 x RCV       0min 44sec \n",
    "PCR         100 x RDCV      2min 56sec \n",
    "PLS         100 x RDCV      2min 27sec \n",
    "PRM         single CV       4min 15sec \n",
    "PRM         20 x RDCV     241min 15sec \n",
    "Ridge       100 x RCV       1min 40sec \n",
    "Lasso       single CV       0min 33sec \n")


###################################################
### chunk number 68: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 69: 
###################################################
  library(gclus)
  data(wine)
  X <- data.frame(scale(wine[,2:14]))   # data without class information
  grp <- as.factor(wine[,1])            # class information
  wine <- data.frame(X=X, grp=grp)
  train <- sample(1:length(grp), round(2/3*length(grp)))


###################################################
### chunk number 70: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 71: 01-knneval
###################################################
  knneval <- knnEval(X, grp, train, knnvec=seq(1,30), legpos="topright")


###################################################
### chunk number 72: 
###################################################
  indmin <- which.min(knneval$cvMean)
  thresh <- knneval$cvMean[indmin] + knneval$cvSe[indmin]
  fvec <- (knneval$cvMean < thresh)
  indopt <- min((1:indmin)[fvec[1:indmin]])
  opt <- knneval$knnvec[indopt]


###################################################
### chunk number 73: 
###################################################
  cat(" optimal number of nearest neighbors:", "k =", opt, "\n",
      "test error at optimum: ", round(knneval$testerr[indopt],4), "\n",
      "CV error threshold:    ", round(thresh,4), "\n", sep=" ")


###################################################
### chunk number 74: 02-repknn eval=FALSE
###################################################
##   res <- array(dim=c(100,6))
##   colnames(res) <- c("k","trainerr","testerr","cvmean","cvse","threshold")
##   tt <- proc.time()
##   for (i in 1:100) {
##     train <- sample(1:length(grp), round(2/3*length(grp)))
##     knneval <- knnEval(X, grp, train, knnvec=seq(1,30), plotit=FALSE)
##     indmin <- which.min(knneval$cvMean)
##     res[i,6] <- knneval$cvMean[indmin] + knneval$cvSe[indmin]
##     fvec <- (knneval$cvMean < res[i,6])
##     indopt <- min((1:indmin)[fvec[1:indmin]])
##     res[i,1] <- knneval$knnvec[indopt]
##     res[i,2] <- knneval$trainerr[indopt]
##     res[i,3] <- knneval$testerr[indopt]
##     res[i,4] <- knneval$cvMean[indopt]
##     res[i,5] <- knneval$cvSe[indopt]
##   }
##   tt <- proc.time() - tt
##   plot(table(res[,1]), xlab="Number of Nearest Neighbors", ylab="Frequency")


###################################################
### chunk number 75: 
###################################################
  mins <- 263.52/60; leftsec <- (mins - floor(mins))*60
  cat("                   median    sd", "\n",
       " training error   0.0169   0.0138 \n", #round(median(res[,2], na.rm=TRUE),4), round(sd(res[,2], na.rm=TRUE),4), "\n",
       " test error       0.05     0.0228 \n", #round(median(res[,3], na.rm=TRUE),4), " ", round(sd(res[,3], na.rm=TRUE),4), "\n",
       " CV error mean    0.0258   0.0136 \n", #round(median(res[,4], na.rm=TRUE),4), round(sd(res[,4], na.rm=TRUE),4), "\n",
       " (computing time for 100 repetitions: 4 min 24 sec) \n", sep=" ") #, floor(mins), "min", round(leftsec,0), "sec) \n", sep=" ")
  #resknn <- res
  timeknn <- round(mins,0)


###################################################
### chunk number 76:  eval=FALSE
###################################################
##   pred <- knn(X[train,], X[-train,], cl=grp[train], k = 1)


###################################################
### chunk number 77: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 78:  eval=FALSE
###################################################
##   tree <- rpart(grp~., data=wine, method="class")
##   plot(tree, main="Full Tree")
##   text(tree)


###################################################
### chunk number 79:  eval=FALSE
###################################################
##   treeeval <- treeEval(X, grp, train, cp=seq(0.45,0.05,by=-0.05))


###################################################
### chunk number 80: 03-treeeval
###################################################
  tree <- rpart(grp~., data=wine, method="class")
  par(mfrow=c(1,2))
  par(mar=c(1,0,4,2))
  plot(tree, main="Full Tree")
  text(tree)
  par(mar=c(5,4,1,2))
  treeeval <- treeEval(X, grp, train, cp=seq(0.45,0.05,by=-0.05), legpos="topright")


###################################################
### chunk number 81: 
###################################################
  indmin <- which.min(treeeval$cvMean)
  thresh <- treeeval$cvMean[indmin] + treeeval$cvSe[indmin]
  fvec <- (treeeval$cvMean < thresh)
  indopt <- min((1:indmin)[fvec[1:indmin]])
  opt <- treeeval$cp[indopt]
  cat(" optimal tree complexity:", "cp =", opt, "\n",
      "test error at optimum: ", round(treeeval$testerr[indopt],4), "\n",
      "CV error threshold:    ", round(thresh,4), "\n", sep=" ")


###################################################
### chunk number 82:  eval=FALSE
###################################################
##   res <- array(dim=c(100,6))
##   colnames(res) <- c("cp","trainerr","testerr","cvmean","cvse","threshold")
##   tt <- proc.time()
##   for (i in 1:100) {
##     print(i)
##     train <- sample(1:length(grp), round(2/3*length(grp)))
##     treeeval <- treeEval(X, grp, train, cp=seq(0.45,0.05,by=-0.05), plotit=FALSE)
##     indmin <- which.min(treeeval$cvMean)
##     res[i,6] <- treeeval$cvMean[indmin] + treeeval$cvSe[indmin]
##     fvec <- (treeeval$cvMean < res[i,6])
##     indopt <- min((1:indmin)[fvec[1:indmin]])
##     res[i,1] <- treeeval$cp[indopt]
##     res[i,2] <- treeeval$trainerr[indopt]
##     res[i,3] <- treeeval$testerr[indopt]
##     res[i,4] <- treeeval$cvMean[indopt]
##     res[i,5] <- treeeval$cvSe[indopt]
##   }
##   tt <- proc.time() - tt


###################################################
### chunk number 83: 
###################################################
  mins <- 450.81/60; leftsec <- (mins - floor(mins))*60
  cat("                   median     sd \n",
       " training error   0.0763   0.0325 \n", #round(median(res[,2], na.rm=TRUE),4), round(sd(res[,2], na.rm=TRUE),4), "\n",
       " test error       0.15     0.045  \n", #round(median(res[,3], na.rm=TRUE),4), " ", round(sd(res[,3], na.rm=TRUE),4), "\n",
       " CV error mean    0.1432   0.04   \n", #round(median(res[,4], na.rm=TRUE),4), round(sd(res[,4], na.rm=TRUE),4), "\n",
       " (computing time for 100 repetitions: 7 min 31 sec) \n", sep=" ") #floor(mins), "min", round(leftsec,0), "sec) \n", sep=" ")
  #restree <- res
  timetree <- round(mins,0)


###################################################
### chunk number 84:  eval=FALSE
###################################################
##   opttree <- prune(tree, cp=0.3)
##   plot(opttree, main="Optimal Tree")
##   text(opttree)


###################################################
### chunk number 85: 04-prunetree eval=FALSE
###################################################
##   par(mfrow=c(1,2))
##   par(mar=c(5,4,1,0))
##   plot(table(res[,1]), xlab="Complexity Parameters", ylab="Frequency")
##   opttree <- prune(tree, cp=0.3)
##   par(mar=c(2,4,4,0))
##   plot(opttree, main="Optimal Tree")
##   text(opttree)


###################################################
### chunk number 86: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 87:  eval=FALSE
###################################################
##   weightsel <- c(0,0.01,0.1,0.15,0.2,0.3,0.5,1)
##   nneteval <- nnetEval(X, grp, train, decay=weightsel, size=5)
##   nneteval <- nnetEval(X, grp, train, decay=0.2, size=seq(5,30,by=5))


###################################################
### chunk number 88:  eval=FALSE
###################################################
##   res <- array(dim=c(100,6))
##   colnames(res) <-
##     c("weight","trainerr","testerr","cvmean","cvse","threshold")
##   tt <- proc.time()
##   for (i in 1:100) {
##     print(i)
##     train <- sample(1:length(grp), round(2/3*length(grp)))
##     nneteval <- nnetEval(X, grp, train, decay=weightsel, size=5, plotit=FALSE)
##     indmin <- which.min(nneteval$cvMean)
##     res[i,6] <- nneteval$cvMean[indmin] + nneteval$cvSe[indmin]
##     fvec <- (nneteval$cvMean < res[i,6])
##     indopt <- min((1:indmin)[fvec[1:indmin]])
##     res[i,1] <- nneteval$decay[indopt]
##     res[i,2] <- nneteval$trainerr[indopt]
##     res[i,3] <- nneteval$testerr[indopt]
##     res[i,4] <- nneteval$cvMean[indopt]
##     res[i,5] <- nneteval$cvSe[indopt]
##   }
##   tt <- proc.time() - tt


###################################################
### chunk number 89: 
###################################################
  mins <- 614.66/60; leftsec <- (mins - floor(mins))*60
  cat("                   median     sd", "\n",
       " training error   0        0.0013 \n", #round(median(res[,2], na.rm=TRUE),4), "    ", round(sd(res[,2], na.rm=TRUE),4), "\n",
       " test error       0.0169   0.0167 \n", #round(median(res[,3], na.rm=TRUE),4), round(sd(res[,3], na.rm=TRUE),4), "\n",
       " CV error mean    0.0167   0.0091 \n", #round(median(res[,4], na.rm=TRUE),4), round(sd(res[,4], na.rm=TRUE),4), "\n",
       " (computing time for 100 repetitions: 10 min 15 sec) \n", sep=" ") #floor(mins), "min", round(leftsec,0), "sec) \n", sep=" ")
  #resnnet <- res
  timennet <- round(mins,0)


###################################################
### chunk number 90: 05-nnetfreq eval=FALSE
###################################################
##   plot(table(res[,1]), xlab="weights", ylab="Frequency")


###################################################
### chunk number 91: 06-nneteval eval=FALSE
###################################################
##   par(mfrow=c(1,2))
##   nneteval <- nnetEval(wine, grp, train, size=5, legpos="topright",
##     decay =  c(0,0.01,0.1,0.15,0.2,0.3,0.5,1))
##   nneteval <- nnetEval(wine, grp, train, decay=0.01, legpos="topright",
##     size = c(5,10,15,20,25,30,40))


###################################################
### chunk number 92:  eval=FALSE
###################################################
##   rule <- nnet(X[train,], class.ind(grp[train]), size=5, entropy=TRUE,
##     decay=0.01)
##   pred <- predict(rule, X[-train,])  # predicted probabilities for test set
##   pred <- apply(pred,1,which.max)    # predicted groups for test set


###################################################
### chunk number 93: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 94:  eval=FALSE
###################################################
##   gamsel <- c(0.001,0.01,0.02,0.05,0.1,0.15,0.2,0.5,1)
##   svmeval <- svmEval(X, grp, train, gamvec=gamsel, kernel="radial",
##     legpos="top")


###################################################
### chunk number 95:  eval=FALSE
###################################################
##   repl <- 100
##   gamsel <- c(0.001,0.01,0.02,0.05,0.1,0.15,0.2,0.5,1)
##   res <- array(dim=c(repl,6))
##   colnames(res) <- c("gamma","trainerr","testerr","cvmean","cvse","threshold")
##   tt <- proc.time()
##   for (i in 1:repl) {
##     train <- sample(1:length(grp), round(2/3*length(grp)))
##     se <- svmEval(X, grp, train, gamvec=gamsel, kernel="radial", plotit=FALSE)
##     indmin <- which.min(se$cvMean)
##     res[i,6] <- se$cvMean[indmin] + se$cvSe[indmin]
##     fvec <- (se$cvMean < res[i,6])
##     indopt <- min((1:indmin)[fvec[1:indmin]])
##     res[i,1] <- se$gamvec[indopt]
##     res[i,2] <- se$trainerr[indopt]
##     res[i,3] <- se$testerr[indopt]
##     res[i,4] <- se$cvMean[indopt]
##     res[i,5] <- se$cvSe[indopt]
##   }
##   tt <- proc.time() - tt


###################################################
### chunk number 96: 08-svmeval eval=FALSE
###################################################
##   par(mfrow=c(1,2))
##   gamsel <- c(0.001,0.01,0.02,0.05,0.1,0.15,0.2,0.5,1)
##   svmeval <- svmEval(X, grp, train, gamvec=gamsel, kernel="radial",
##     legpos="top")
##   plot(table(res[,1]), xlab="Gamma", ylab="Frequency")


###################################################
### chunk number 97: 
###################################################
  mins <- 435.87/60; leftsec <- (mins - floor(mins))*60
  cat("                   median     sd", "\n",
       " training error   0.0085   0.0061 \n", #round(median(res[,2], na.rm=TRUE),4), round(sd(res[,2], na.rm=TRUE),4), "\n",
       " test error       0.0167   0.0145 \n", #round(median(res[,3], na.rm=TRUE),4), round(sd(res[,3], na.rm=TRUE),4), "\n",
       " CV error mean    0.0167   0.0081 \n", #round(median(res[,4], na.rm=TRUE),4), round(sd(res[,4], na.rm=TRUE),4), "\n",
       " (computing time for 100 repetitions: 7 min 16 sec) \n", sep=" ") #floor(mins), "min", round(leftsec,0), "sec) \n", sep=" ")
 #ressvm <- res
  timesvm <- round(mins,0)


###################################################
### chunk number 98:  eval=FALSE
###################################################
##   rule <- svm(X[train,],grp[train], kernel="radial", gamma=0.01)
##   pred <- predict(svmres, X[-train,])


###################################################
### chunk number 99: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 100: 09-classcomp eval=FALSE
###################################################
##   par(mar=c(2,4,2,1))
##   resdat <- data.frame(kNN=resknn[,3],Tree=restree[,3],ANN=resnnet[,3],SVM=ressvm[,3])
##   boxplot(resdat,ylab="Test error", las=1)


###################################################
### chunk number 101: 
###################################################
  cat("Method   Median test error   Computing time \n \n",
      "kNN            0.05              4 min \n", #round(median(resknn[,3], na.rm=TRUE),3),  "              ", timeknn,  " min \n",
      "Tree           0.15              8 min \n", #round(median(restree[,3], na.rm=TRUE),3), "              ", timetree, " min \n",
      "ANN            0.017            10 min \n", #round(median(resnnet[,3], na.rm=TRUE),3), "            ", timennet,   " min \n",
      "SVM            0.017             7 min \n", sep="") #round(median(ressvm[,3], na.rm=TRUE),3),  "             ", timesvm,   " min \n", sep="")


###################################################
### chunk number 102: 
###################################################
  options(prompt="> ", continue="  ")
  options(width=70)


###################################################
### chunk number 103:  eval=FALSE
###################################################
##   X_alr <- alr(X, divisorvar=2)


###################################################
### chunk number 104:  eval=FALSE
###################################################
##   X_clr <- clr(X)


###################################################
### chunk number 105:  eval=FALSE
###################################################
##   X_ilr <- ilr(X)


